/**
 * Persona Controller.
 *
 * Responde a eventos referente a la CLASE PERSONA ,invoca peticiones al
 * 'modelo' cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un documento o un registro en una base de datos).
 *
 * También puede enviar comandos a su 'vista' asociada si se solicita un cambio
 * en la forma en que se presenta de 'modelo' (por ejemplo, desplazamiento o
 * scroll por un documento o por los diferentes registros de una base de datos),
 * por tanto se podría decir que el 'controlador' hace de intermediario entre la
 * 'vista' y el 'modelo'.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Throws in DAO
 */
package controller;

import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.ModelDriven;
import java.util.ArrayList;
import java.util.Iterator;
import model.dao.DocumentoDAO;
import model.dao.PersonaDAO;
import model.entities.Documento;
import model.entities.Persona;
import model.pdf.ObjectPdf;
import model.util.MessageUtil;

public class PersonaController extends ActionSupport implements ModelDriven<Persona> {

    private Persona persona = new Persona();
    private ArrayList<Persona> listPersona = new ArrayList<Persona>();
    private PersonaDAO personaDAO = new PersonaDAO();
    private Integer currentUser = null;
    private String option = null;
    private MessageUtil message = new MessageUtil();
    private String msg = null;

    /**
     * Implementacion del ModelDriven
     *
     * Permite enlazar tags de Struts 2 con el objeto del Action
     *
     * @return Retorna el Objeto del ModelDriven
     */
    @Override
    public Persona getModel() {
        return persona;
    }

    /**
     * Getter del Usuario Logueado
     *
     * @return Retorna el Usuario Logueado
     */
    public Integer getCurrentUser() {
        return currentUser;
    }

    /**
     * Setter del Usuario Logeado
     *
     * @param current Retorna el Usuario Logueado.
     */
    public void setCurrentUser(Integer currentUser) {
        this.currentUser = currentUser;
    }

    /**
     * Getter del Objeto Persona
     *
     * @return Retorna el incidencia
     */
    public Persona getPersona() {
        return persona;
    }

    /**
     * Setter del Objeto Persona
     *
     * @param incidencia Assignala Persona
     */
    public void setPersona(Persona persona) {
        this.persona = persona;
    }

    /**
     * Getter de la lista de Personas
     *
     * @return Retorna la lista de Personas
     */
    public ArrayList<Persona> getListPersona() {
        return listPersona;
    }

    /**
     * Setter de la lista de Personas
     *
     * @param listPersona Assigna la lista de Personas
     */
    public void setListPersona(ArrayList<Persona> listPersona) {
        this.listPersona = listPersona;
    }

    /**
     * Getter de Mensage
     *
     * @return Retorna el mensage destinado al Usuario
     */
    public String getMsg() {
        return msg;
    }

    /**
     * Setter de Mensage
     *
     * @param msg Mensage destinado al Usuario
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

    /**
     * Getter de Option
     *
     * @return
     */
    public String getOption() {
        return option;
    }

    /**
     * Setter de Option
     *
     * @param option Opcion Elegida por el Usuario
     */
    public void setOption(String option) {
        this.option = option;
    }

    /**
     * Insertar Persona
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String insertPersona() {

        if (this.validateInsert()) {
            if (personaDAO.insertPersona(persona)) {
                msg = "La Persona ha sido Insertada Correctamente.";

                persona = new Persona();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Insertar la Persona.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona la Persona
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectPersona() {

        if (this.validateSelect()) {

            if (option.equals("Buscar por ID")) {
                persona = personaDAO.selectPersona(persona.getIdPersona());
                if (persona != null) {
                    listPersona.add(persona);
                }

            } else if (option.equals("Buscar por Documento")) {

                DocumentoDAO docDAO = new DocumentoDAO();
                Documento doc = docDAO.selectDocumento(persona.getDocumento());

                if (doc != null) {
                    persona = personaDAO.selectPersona(doc);

                    if (persona != null) {
                        persona.setDocumento(doc);
                    }
                    listPersona.add(persona);
                }
            } else {
                listPersona = personaDAO.selectByExample(persona);
            }

            if (listPersona.size() > 0) {
                persona = listPersona.get(0);
                msg = "La Persona se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                this.persona = null;
                msg = "Ha Ocurrido un Error al Consultar la Persona.";
                addActionError(msg);
                msg = "La Persona no Existe.";
                addFieldError("", msg);
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            this.persona = null;
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona todas las Personas
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectAllPersona() {
        listPersona = personaDAO.selectAll();
        if (listPersona.size() > 0) {
            msg = "Las Personas se ha Seleccionado Correctamente.";
        } else {
            msg = "La base de datos de Personas esta Vacia.";
            message = new MessageUtil(msg);
            return "KO";
        }
        return "OK";
    }

    /**
     * Selecciona las Personas que coinciden con el Ejemplo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectByExamplePersona() {

        if (this.validateExample()) {

            ArrayList<Persona> lp = personaDAO.selectByExample(persona);

            if (persona.getDocumento() != null) {

                Iterator<Persona> it = lp.listIterator();
                while (it.hasNext()) {
                    Persona per = it.next();

                    if (per.getDocumento().getTipoDocumento().equals(persona.getDocumento().getTipoDocumento())) {
                        listPersona.add(per);
                    }
                }
            } else {
                listPersona = lp;
            }

            if (listPersona.size() > 0) {
                msg = "La Persona se ha Seleccionado Correctamente.";
                return "OK";
            } else {

                msg = "No Existen Personas con esas Caracteristicas.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Estan todos los campos vacios.";
            return this.selectAllPersona();
        }
    }

    /**
     * Elimina la Persona
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String deletePersona() {
        msg = "Desea Eliminar el Persona??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (personaDAO.deletePersona(persona)) {
                msg = "La Persona ha sido Eliminado Correctamente.";
                message = new MessageUtil(msg);
                persona = new Persona();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Eliminar la Persona.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Modifica la Persona
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String updatePersona() {
        msg = "Desea Modificar el Persona??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (personaDAO.updatePersona(persona)) {
                msg = "El Persona ha sido Modificado Correctamente.";
                message = new MessageUtil(msg);
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Modificado la Persona.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Imprime una persona
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printPersona() {

        if (persona != null) {
            ObjectPdf objPdf = new ObjectPdf(this.persona);
            return "OK";
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Personas
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printPersonas() {
        if (this.selectByExamplePersona().equals("OK")) {

            if (listPersona.size() > 0) {
                ObjectPdf objPdf = new ObjectPdf(this.listPersona);
                return "OK";
            } else {
                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Personas
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printAllPersonas() {
        if (this.selectAllPersona().equals("OK")) {

            if (listPersona.size() > 0) {
                ObjectPdf objPdf = new ObjectPdf(this.listPersona);
                return "OK";
            } else {
                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    //-------------------------------------------------------------
    //------------------------VALIDATES----------------------------
    //-------------------------------------------------------------
    /**
     * Valida los campos de Insercion de datos
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateInsert() {
        boolean bool = true;

        if (this.persona.getDocumento().getNumeroDocumento().equals("")) {
            addFieldError("", "El campo Documento es Obligatorio");
            bool = false;
        }

        if (this.persona.getNombre().equals("")) {
            addFieldError("", "El campo Nombre es Obligatorio");
            bool = false;
        }

        if (this.persona.getApellido1().equals("")) {
            addFieldError("", "El campo Apellido 1 es Obligatorio");
            bool = false;
        }

        if (this.persona.getApellido2().equals("")) {
            addFieldError("", "El campo Apellido 2 es Obligatorio");
            bool = false;
        }

        if (this.persona.getTelefono().equals("")) {
            addFieldError("", "El campo Telefono es Obligatorio");
            bool = false;
        }

        if (this.persona.getEmail().equals("")) {
            addFieldError("", "El campo Email es Obligatorio");
            bool = false;
        }
        return bool;
    }

    /**
     * Valida los Campos de seleccion de la Persona
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateSelect() {
        boolean bool = true;

        persona.onlyParameter(option);

        if (option.equals("Buscar por ID")) {
            if (this.persona.getIdPersona() == null) {
                addFieldError("", "Introduzca el ID");
                bool = false;
            }
        }

        if (option.equals("Buscar por Documento")) {
            if (this.persona.getDocumento().getNumeroDocumento().equals("")) {
                addFieldError("", "Introduzca el Documento");
                bool = false;
            }
        }

        if (option.equals("Buscar por Telefono")) {
            if (this.persona.getTelefono().equals("")) {
                addFieldError("", "Introduzca el Telefono");
                bool = false;
            }
        }

        if (option.equals("Buscar por Email")) {
            if (this.persona.getEmail().equals("")) {
                addFieldError("", "Introduzca el Email");
                bool = false;
            }
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la validacion de Personas elegidas por Ejemplo
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateExample() {
        boolean bool = true;
        int n = 0;

        if (this.persona.getIdioma().equals("")) {
            n = n + 1;
            persona.setIdioma(null);
        }

        if (this.persona.getDocumento().getTipoDocumento().equals("")) {
            n = n + 1;
            persona.setDocumento(null);
        }

        if (this.persona.getNombre().equals("")) {
            n = n + 1;
            persona.setNombre(null);
        }

        if (this.persona.getApellido1().equals("")) {
            n = n + 1;
            persona.setApellido1(null);
        }


        if (this.persona.getApellido2().equals("")) {
            n = n + 1;
            persona.setApellido2(null);
        }

        if (n == 5) {
            bool = false;
        }
        return bool;
    }

    //-------------------------------------------------------------
    //--------------------------PANTALLAS--------------------------
    //-------------------------------------------------------------
    /**
     * Abrir Pantalla de Seleccion de Persona
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectPersona() {
        persona = new Persona();
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Modificacion de Persona
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenUpdatePersona() {
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Persona mediante Ejemplo
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectByExamplePersona() {
        this.persona = new Persona();
        return "PANTALLA";
    }
}
