/**
 * Plantilla Controller.
 *
 * Responde a eventos referente a la CLASE PLANTILLA ,invoca peticiones al
 * 'modelo' cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un documento o un registro en una base de datos).
 *
 * También puede enviar comandos a su 'vista' asociada si se solicita un cambio
 * en la forma en que se presenta de 'modelo' (por ejemplo, desplazamiento o
 * scroll por un documento o por los diferentes registros de una base de datos),
 * por tanto se podría decir que el 'controlador' hace de intermediario entre la
 * 'vista' y el 'modelo'.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Throws in DAO
 */
package controller;

import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.ModelDriven;
import java.util.ArrayList;
import model.dao.PlantillaDAO;
import model.entities.Plantilla;
import model.pdf.ObjectPdf;
import model.util.MessageUtil;

public class PlantillaController extends ActionSupport implements ModelDriven<Plantilla> {

    private Plantilla plantilla = new Plantilla();
    private ArrayList<Plantilla> listPlantilla = new ArrayList<Plantilla>();
    private String option = null;
    private MessageUtil message = new MessageUtil();
    private PlantillaDAO plantillaDAO = new PlantillaDAO();
    private String msg = null;

    /**
     * Implementacion del ModelDriven
     *
     * Permite enlazar tags de Struts 2 con el objeto del Action
     *
     * @return Retorna el Objeto del ModelDriven
     */
    @Override
    public Plantilla getModel() {
        return plantilla;
    }

    /**
     * Getter del Objeto Plantilla
     *
     * @return Retorna el plantilla
     */
    public Plantilla getPlantilla() {
        return plantilla;
    }

    /**
     * Setter del Objeto Plantilla
     *
     * @param plantilla Assigna la Plantilla
     */
    public void setPlantilla(Plantilla plantilla) {
        this.plantilla = plantilla;
    }

    /**
     * Getter de la lista de Plantillas
     *
     * @return Retorna la lista de Plantillas
     */
    public ArrayList<Plantilla> getListPlantilla() {
        return listPlantilla;
    }

    /**
     * Setter de la lista de Plantillas
     *
     * @param listPlantilla Assigna la lista de Plantillas
     */
    public void setListPlantilla(ArrayList<Plantilla> listPlantilla) {
        this.listPlantilla = listPlantilla;
    }

    /**
     * Getter de Mensage
     *
     * @return Retorna el mensage destinado al Usuario
     */
    public String getMsg() {
        return msg;
    }

    /**
     * Setter de Mensage
     *
     * @param msg Mensage destinado al Usuario
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

    /**
     * Getter de Option
     *
     * @return
     */
    public String getOption() {
        return option;
    }

    /**
     * Setter de Option
     *
     * @param option Opcion Elegida por el Usuario
     */
    public void setOption(String option) {
        this.option = option;
    }

    /**
     * Insertar Plantilla
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String insertPlantilla() {
        if (this.validateInsert()) {

            if (plantillaDAO.insertPlantilla(plantilla)) {
                plantilla = new Plantilla();
                msg = "La Plantilla ha sido Insertado Correctamente.";
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Insertar la Plantilla.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona la Plantilla
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectPlantilla() {
        if (this.validateSelect()) {

            if (option.equals("Buscar por ID")) {
                plantilla = plantillaDAO.selectPlantilla(plantilla.getIdPlantilla());
                if (plantilla != null) {
                    listPlantilla.add(plantilla);
                }
            } else {
                listPlantilla = plantillaDAO.selectByExample(plantilla);
            }

            if (listPlantilla.size() > 0) {

                plantilla = listPlantilla.get(0);
                msg = "El Plantilla se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                msg = "El Plantilla no Existe.";
                message = new MessageUtil(msg);
                addActionError(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona todas las Plantillas
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectAllPlantilla() {
        listPlantilla = plantillaDAO.selectAll();
        if (listPlantilla.size() > 0) {
            msg = "Los Plantillas se ha Seleccionado Correctamente.";
        } else {
            msg = "La base de datos de Plantillas esta Vacia.";
            message = new MessageUtil(msg);
            return "KO";
        }
        return "OK";
    }

    /**
     * Selecciona las Plantillas que coinciden con el Ejemplo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectByExamplePlantilla() {
        if (this.validateExample()) {
            listPlantilla = plantillaDAO.selectByExample(plantilla);

            if (listPlantilla.size() > 0) {
                msg = "El Plantilla se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                msg = "No Existen Plantillas con esas Caracteristicas.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Todos los Campos estan Vacios.";
            return this.selectAllPlantilla();
        }
    }

    /**
     * Elimina la Plantilla
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String deletePlantilla() {
        msg = "Desea Eliminar la Plantilla??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (plantillaDAO.deletePlantilla(plantilla)) {
                msg = "El Plantilla ha sido Eliminado Correctamente.";
                message = new MessageUtil(msg);
                plantilla = new Plantilla();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Eliminar la Plantilla.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Modifica la Plantilla
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String updatePlantilla() {
        msg = "Desea Modificar la Plantilla??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (plantillaDAO.updatePlantilla(plantilla)) {
                msg = "El Plantilla ha sido Modificado Correctamente.";
                message = new MessageUtil(msg);
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Modificado la Plantilla.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Imprime una incidencia
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printPlantilla() {
        if (plantilla != null) {
            ObjectPdf objPdf = new ObjectPdf(this.plantilla);

            return "OK";
        } else {

            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Plantillas
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printPlantillas() {
        if (this.selectByExamplePlantilla().equals("OK")) {
            if (listPlantilla != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listPlantilla);

                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Plantillas
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printAllPlantillas() {
        if (this.selectAllPlantilla().equals("OK")) {
            if (listPlantilla != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listPlantilla);

                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }
    //-------------------------------------------------------------
    //------------------------VALIDATES----------------------------
    //-------------------------------------------------------------

    /**
     * Valida los campos de Insercion de datos
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateInsert() {

        if (this.plantilla.getTipo().equals("")) {
            addFieldError("", "El campo Tipo es Obligatorio");
            return false;
        }

        if (this.plantilla.getTamano().equals("")) {
            addFieldError("", "El campo Tamaño es Obligatorio");
            return false;
        }

        Plantilla plan = plantillaDAO.selectPlantilla(plantilla);
        if (plan != null) {
            addFieldError("", "La Plantilla Ya Existe.");
            plantilla = new Plantilla();
            return false;
        }
        return true;
    }

    /**
     * Valida los Campos de seleccion de la Plantilla
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateSelect() {
        boolean bool = true;

        plantilla.onlyParameter(option);

        if (option.equals("Buscar por ID")) {
            if (this.plantilla.getIdPlantilla() == null) {
                addFieldError("", "Introduzca el ID");
                bool = false;
            }
        }

        if (option.equals("Buscar Tipo/Tamaño")) {
            if (this.plantilla.getTipo().equals("")) {
                addFieldError("", "Introduzca el Tipo");
                bool = false;
            }
            if (this.plantilla.getTamano().equals("")) {
                addFieldError("", "Introduzca el Tamano");
                bool = false;
            }
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la validacion de Plantillas elegidas por Ejemplo
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateExample() {
        boolean bool = true;
        int n = 0;

        if (this.plantilla.getTipo().equals("")) {
            n = n + 1;
            plantilla.setTipo(null);
        }

        if (this.plantilla.getTamano().equals("")) {
            n = n + 1;
            plantilla.setTamano(null);
        }

        if (n == 2) {
            bool = false;
        }
        return bool;
    }

    //-------------------------------------------------------------
    //--------------------------PANTALLAS--------------------------
    //-------------------------------------------------------------
    /**
     * Abrir Pantalla de Seleccion de Plantilla
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectPlantilla() {
        plantilla = new Plantilla();
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Modificacion de Plantilla
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenUpdatePlantilla() {
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Plantilla mediante Ejemplo
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectByExamplePlantilla() {
        plantilla = new Plantilla();
        return "PANTALLA";
    }
}
