/**
 * ArticuloDAO.
 *
 * La capa Modelo: Es la representación específica de la información con la cual
 * el sistema opera, por lo tanto gestiona todos los accesos a dicha
 * información, tanto consultas como actualizaciones, implementando también los
 * privilegios de acceso que se hayan descrito en las especificaciones de la
 * aplicación (lógica de negocio).
 *
 * Envía a la 'vista' aquella parte de la información que en cada momento se le
 * solicita para que sea mostrada (típicamente a un usuario). Las peticiones de
 * acceso o manipulación de información llegan al 'modelo' a través del
 * 'controlador'.
 *
 * La parte DAO del sistema responde a las peticiones de referentes a la Base de
 * Datos cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un registro o insertar en una base de datos).
 *
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Como lo que se busca es caputar la excepcion para poder tratarla y
 * que el Usuario pueda seguir utilizando la aplicacion con total normalidad, se
 * captura la excepcion general "Exception" ya que en todos los casos el
 * tratamiento de la excepcion es el mismo.
 */
package model.dao;

import java.util.ArrayList;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Restrictions;
import model.entities.Articulo;
import org.hibernate.Transaction;

public class ArticuloDAO {

    Session session;
    Transaction transaction;

    /**
     * Constructor Vacio.
     */
    public ArticuloDAO() {
    }

    /**
     * Funcion que Abre la Sesion
     * 
     * @return Sesion Abierta
     */
    private Session getSession() {
        try {
            session = HibernateUtil.getSessionFactory().openSession();
            return session;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Funcion que obtiene la transaccion
     * 
     * @return transaccion abierta
     */
    private Transaction beginTransaction() {

        try {
            transaction = session.beginTransaction();
            return transaction;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Insertar Articulo
     * 
     * Inserta el Articulo en la BD.
     * @param art Articulo a Insertar.
     * @return Retorna True o False dependiendo del Exito del procedimiento.
     */
    public boolean insertArticulo(Articulo art) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.save(art);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    //selects with criteria
    /**
     * Seleccion del Articulo Mediante ID Articulo
     * 
     * @param id ID del Articulo a Seleccionar
     * @return Retorna el Articulo en caso de Exito. En caso de Excepcion Null
     */
    public Articulo selectArticulo(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Articulo.class);
            criteria.add(Restrictions.eq("idArticulo", id));

            ArrayList<Articulo> result = (ArrayList<Articulo>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Elimina el objeto de la BD
     * 
     * @param art Articulo a Eliminar
     * @return Retorna el Articulo en caso de Exito. En caso de Excepcion Null
     */
    public boolean deleteArticulo(Articulo art) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.delete(art);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Modifica el Articulo de la BD
     * 
     * @param art Articulo a Modificar
     * @return Retorna el Articulo en caso de Exito. En caso de Excepcion Null
     */
    public boolean updateArticulo(Articulo art) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.update(art);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Selecciona todos los Articulos de la BD
     * 
     * @return Retorna una lista del Articulo en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Articulo> selectAll() {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();
            Criteria myCriteria = session.createCriteria(Articulo.class);
            ArrayList<Articulo> result = (ArrayList<Articulo>) myCriteria.list();
            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Selecciona los Articulos mediante Un Ejemplo
     * 
     * @param example Articulo de Exemplo a seleccionar
     * @return Retorna una lista del Articulo en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Articulo> selectByExample(Articulo example) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria myCriteria = session.createCriteria(Articulo.class);
            myCriteria.add(Example.create(example));

            return (ArrayList<Articulo>) myCriteria.list();
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion del Articulo Mediante Codigo Ean
     * 
     * @param art Articulo a Seleccionar
     * @return Retorna el Articulo en caso de Exito. En caso de Excepcion Null
     */
    public Articulo selectArticulo(Articulo art) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Articulo.class);
            criteria.add(Restrictions.eq("ean", art.getEan()));

            ArrayList<Articulo> result = (ArrayList<Articulo>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Seleccion del Articulo Mediante tecla
     * 
     * @param tecla Tecla del Articulo a Seleccionar
     * @return Retorna el Articulo en caso de Exito. En caso de Excepcion Null
     */
    public Articulo selectArticuloTecla(String tecla) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Articulo.class);
            criteria.add(Restrictions.eq("tecla", tecla));

            ArrayList<Articulo> result = (ArrayList<Articulo>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Seleccion del Articulo Mediante Codigo Sms
     * 
     * @param sms Codigo Sms del Articulo a Seleccionar
     * @return Retorna el Articulo en caso de Exito. En caso de Excepcion Null
     */
    public Articulo selectArticuloSms(String sms) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Articulo.class);
            criteria.add(Restrictions.eq("codInterno", sms));

            ArrayList<Articulo> result = (ArrayList<Articulo>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            return null;
        }
    }

    //modifications with HQL
        /**
     * Elimina el objeto de la BD meidante ID
     * 
     * @param id ID del Articulo a Eliminar
     * @return Retorna el Articulo en caso de Exito. En caso de Excepcion Null
     */
    public boolean deleteArticulo(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            String hql = "delete from Articulo where idArticulo = :id";
            Query query = session.createQuery(hql);
            query.setInteger("id", id);

            int rowCount = query.executeUpdate();
            System.out.println("Rows affected: " + rowCount);

            session.getTransaction().commit();
            session.close();

            return true;
        } catch (Exception e) {
            session.getTransaction().rollback();
            return false;
        }
    }

    /**
     * Elimina el objeto de la BD mediante Codigo Ean
     * 
     * @param ean Codigo Ean del Articulo a Eliminar
     * @return Retorna el Articulo en caso de Exito. En caso de Excepcion Null
     */
    public boolean deleteArticulo(String ean) {

        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            String hql = "delete from Articulo where ean = :ean";

            Query query = session.createQuery(hql);
            query.setString("ean", ean);

            int rowCount = query.executeUpdate();
            System.out.println("Rows affected: " + rowCount);

            session.getTransaction().commit();
            session.close();

            return true;
        } catch (Exception e) {
            session.getTransaction().rollback();
            return false;
        }
    }
}
