/**
 * CartelDAO.
 *
 * La capa Modelo: Es la representación específica de la información con la cual
 * el sistema opera, por lo tanto gestiona todos los accesos a dicha
 * información, tanto consultas como actualizaciones, implementando también los
 * privilegios de acceso que se hayan descrito en las especificaciones de la
 * aplicación (lógica de negocio).
 *
 * Envía a la 'vista' aquella parte de la información que en cada momento se le
 * solicita para que sea mostrada (típicamente a un usuario). Las peticiones de
 * acceso o manipulación de información llegan al 'modelo' a través del
 * 'controlador'.
 *
 * La parte DAO del sistema responde a las peticiones de referentes a la Base de
 * Datos cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un registro o insertar en una base de datos).
 *
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Como lo que se busca es caputar la excepcion para poder tratarla y
 * que el Usuario pueda seguir utilizando la aplicacion con total normalidad, se
 * captura la excepcion general "Exception" ya que en todos los casos el
 * tratamiento de la excepcion es el mismo.
 */
package model.dao;

import java.util.ArrayList;
import model.entities.Articulo;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Restrictions;
import model.entities.Cartel;
import model.entities.Plantilla;
import org.hibernate.Hibernate;
import org.hibernate.Transaction;

public class CartelDAO {
    Session session;
    Transaction transaction;

    /**
     * Constructor Vacio.
     */
    public CartelDAO() {
    }

    /**
     * Funcion que Abre la Sesion
     * 
     * @return Sesion Abierta
     */
    private Session getSession() {
        try {
            session = HibernateUtil.getSessionFactory().openSession();
            return session;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Funcion que obtiene la transaccion
     * 
     * @return transaccion abierta
     */
    private Transaction beginTransaction() {

        try {
            transaction = session.beginTransaction();
            return transaction;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Insertar Cartel
     * 
     * Inserta el Cartel en la BD.
     * @param car Cartel a Insertar.
     * @return Retorna True o False dependiendo del Exito del procedimiento.
     */
    public boolean insertCartel(Cartel car) {
        try {

            session = this.getSession();
            transaction = this.beginTransaction();

            session.saveOrUpdate(car);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }


    //selects with criteria
    /**
     * Seleccion del Cartel Mediante ID Cartel
     * 
     * @param id ID del Cartel a Seleccionar
     * @return Retorna el Cartel en caso de Exito. En caso de Excepcion Null
     */
    public Cartel selectCartel(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Cartel.class);
            criteria.add(Restrictions.eq("idCartel", id));

            ArrayList<Cartel> result = (ArrayList<Cartel>) criteria.list();
            if (result.size() > 0) {
                Hibernate.initialize(result.get(0).getArticulo());
                Hibernate.initialize(result.get(0).getPlantilla());

                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    //selects with criteria
    /**
     * Seleccion del Cartel Mediante ID Cartel
     * 
     * @param car cartel a seleccionar
     * @return Retorna el Cartel en caso de Exito. En caso de Excepcion Null
     */
    public Cartel selectCartel(Cartel car) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Cartel.class);
            criteria.add(Restrictions.eq("plantilla.idPlantilla", car.getPlantilla().getIdPlantilla()));
            criteria.add(Restrictions.eq("articulo.idArticulo", car.getArticulo().getIdArticulo()));
            criteria.add(Restrictions.eq("dataIni", car.getDataIni()));
            criteria.add(Restrictions.eq("dataFi", car.getDataFi()));

            ArrayList<Cartel> result = (ArrayList<Cartel>) criteria.list();
            if (result.size() > 0) {
                Hibernate.initialize(result.get(0).getArticulo());
                Hibernate.initialize(result.get(0).getPlantilla());

                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }
    
    /**
     * Seleccion de Cartels Mediante plantilla
     * 
     * @param plan Plantilla del Cartel a Seleccionar
     * @return Retorna el Cartel en caso de Exito. En caso de Excepcion Null
     */
    public Cartel selectCartel(Plantilla plan) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Cartel.class);
            criteria.add(Restrictions.eq("plantilla.idPlantilla", plan.getIdPlantilla()));
            
            ArrayList<Cartel> result = (ArrayList<Cartel>) criteria.list();
            if (result.size() > 0) {
                Hibernate.initialize(result.get(0).getArticulo());
                Hibernate.initialize(result.get(0).getPlantilla());

                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }
    
    /**
     * Seleccion de Cartels Mediante plantilla
     * 
     * @param plan Plantilla del Cartel a Seleccionar
     * @param art Plantilla del Cartel a Seleccionar
     * @return Retorna el Cartel en caso de Exito. En caso de Excepcion Null
     */
    public Cartel selectCartel(Plantilla plan,Articulo art) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Cartel.class);
            criteria.add(Restrictions.eq("plantilla.idPlantilla", plan.getIdPlantilla()));
            criteria.add(Restrictions.eq("articulo.idArticulo", art.getIdArticulo()));
            
            ArrayList<Cartel> result = (ArrayList<Cartel>) criteria.list();
            
            if (result.size() > 0) {
                Hibernate.initialize(result.get(0).getArticulo());
                Hibernate.initialize(result.get(0).getPlantilla());
            }
                return result.get(0);
            
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion del Cartel Mediante ID Cartel
     * 
     * @param art ID del aCartel a Seleccionar
     * @return Retorna el Cartel en caso de Exito. En caso de Excepcion Null
     */
    public Cartel selectCartel(Articulo art) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Cartel.class);
            criteria.add(Restrictions.eq("articulo.idArticulo", art.getIdArticulo()));

            ArrayList<Cartel> result = (ArrayList<Cartel>) criteria.list();
            
            if (result.size() > 0) {
                Hibernate.initialize(result.get(0).getArticulo());
                Hibernate.initialize(result.get(0).getPlantilla());

                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Elimina el objeto de la BD
     * 
     * @param car Cartel a Eliminar
     * @return Retorna el Cartel en caso de Exito. En caso de Excepcion Null
     */
    public boolean deleteCartel(Cartel car) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            String hql = "delete from Cartel where idCartel = :idCartel";

            Query query = session.createQuery(hql);
            query.setInteger("idCartel", car.getIdCartel());

            int rowCount = query.executeUpdate();
            System.out.println("Rows affected: " + rowCount);

            transaction.commit();
            session.close();

            return true;
        } catch (Exception e) {
            session.getTransaction().rollback();
            return false;
        }
    }

    /**
     * Modifica el Cartel de la BD
     * 
     * @param car Cartel a Modificar
     * @return Retorna el Cartel en caso de Exito. En caso de Excepcion Null
     */
    public boolean updateCartel(Cartel car) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.update(car.getPlantilla());

            session.update(car.getArticulo());
            session.update(car);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Selecciona todos los Cartels de la BD
     * 
     * @return Retorna una lista del Cartel en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Cartel> selectAll() {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria myCriteria = session.createCriteria(Cartel.class);
            ArrayList<Cartel> result = (ArrayList<Cartel>) myCriteria.list();

            if (result.size() > 0) {
                for (Cartel l : result) {
                    Hibernate.initialize(l.getArticulo());
                    Hibernate.initialize(l.getPlantilla());
                }
            }
                        System.out.println(result.toString());
            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Selecciona los Cartels mediante Un Ejemplo
     * 
     * @param example Cartel de Exemplo a seleccionar
     * @return Retorna una lista del Cartel en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Cartel> selectByExample(Cartel example) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria myCriteria = session.createCriteria(Cartel.class);
            myCriteria.add(Example.create(example));

            ArrayList<Cartel> result = (ArrayList<Cartel>) myCriteria.list();

            if (result.size() > 0) {
                for (Cartel l : result) {
                    Hibernate.initialize(l.getArticulo());
                    Hibernate.initialize(l.getPlantilla());
                }
            }

            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }
}
