/**
 * DepartamentoDAO.
 *
 * La capa Modelo: Es la representación específica de la información con la cual
 * el sistema opera, por lo tanto gestiona todos los accesos a dicha
 * información, tanto consultas como actualizaciones, implementando también los
 * privilegios de acceso que se hayan descrito en las especificaciones de la
 * aplicación (lógica de negocio).
 *
 * Envía a la 'vista' aquella parte de la información que en cada momento se le
 * solicita para que sea mostrada (típicamente a un usuario). Las peticiones de
 * acceso o manipulación de información llegan al 'modelo' a través del
 * 'controlador'.
 *
 * La parte DAO del sistema responde a las peticiones de referentes a la Base de
 * Datos cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un registro o insertar en una base de datos).
 *
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Como lo que se busca es caputar la excepcion para poder tratarla y
 * que el Usuario pueda seguir utilizando la aplicacion con total normalidad, se
 * captura la excepcion general "Exception" ya que en todos los casos el
 * tratamiento de la excepcion es el mismo.
 */
package model.dao;

import java.util.ArrayList;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Restrictions;
import model.entities.Departamento;
import org.hibernate.Transaction;

public class DepartamentoDAO {

    Session session;
    Transaction transaction;

    /**
     * Constructor Vacio.
     */
    public DepartamentoDAO() {
    }

    /**
     * Funcion que Abre la Sesion
     *
     * @return Sesion Abierta
     */
    private Session getSession() {
        try {
            session = HibernateUtil.getSessionFactory().openSession();
            return session;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Funcion que obtiene la transaccion
     *
     * @return transaccion abierta
     */
    private Transaction beginTransaction() {

        try {
            transaction = session.beginTransaction();
            return transaction;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Insertar Articulo
     *
     * Inserta el Departamento en la BD.
     *
     * @param dep Departamento a Insertar.
     * @return Retorna True o False dependiendo del Exito del procedimiento.
     */
    public boolean insertDepartamento(Departamento dep) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.save(dep);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Seleccion del Departamento Mediante Departamento
     *
     * @param dep Departamento del Departamento a Seleccionar
     * @return Retorna el Departamento en caso de Exito. En caso de Excepcion Null
     */
    public Departamento selectDepartamentoDep(String dep) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Departamento.class);
            criteria.add(Restrictions.eq("departamento", dep));

            ArrayList<Departamento> result = (ArrayList<Departamento>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion del Departamento Mediante Codigo Departamento
     *
     * @param cod Codigo Departamento del Departamento a Seleccionar
     * @return Retorna el Departamento en caso de Exito. En caso de Excepcion Null
     */
    public Departamento selectDepartamentoCod(String cod) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Departamento.class);
            criteria.add(Restrictions.eq("codigoDepartamento", cod));

            ArrayList<Departamento> result = (ArrayList<Departamento>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion del Departamento Mediante ID Departamento
     *
     * @param id ID del Departamento a Seleccionar
     * @return Retorna el Departamento en caso de Exito. En caso de Excepcion Null
     */
    public Departamento selectDepartamento(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Departamento.class);
            criteria.add(Restrictions.eq("idDepartamento", id));

            ArrayList<Departamento> result = (ArrayList<Departamento>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Elimina el objeto de la BD
     *
     * @param dep Departamento a Eliminar
     * @return Retorna el Departamento en caso de Exito. En caso de Excepcion Null
     */
    public boolean deleteDepartamento(Departamento dep) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.delete(dep);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Modifica el Departamento de la BD
     *
     * @param dep Departamento a Modificar
     * @return Retorna el Departamento en caso de Exito. En caso de Excepcion Null
     */
    public boolean updateDepartamento(Departamento dep) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.update(dep);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Selecciona todos los Departamentos de la BD
     *
     * @return Retorna una lista del Departamento en caso de Exito. En caso de
     * Excepcion Null
     */
    public ArrayList<Departamento> selectAll() {
        try {
            Criteria myCriteria = this.getSession().createCriteria(Departamento.class);
            ArrayList<Departamento> result = (ArrayList<Departamento>) myCriteria.list();
            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Selecciona los Departamentos mediante Un Ejemplo
     *
     * @param example Departamento de Exemplo a seleccionar
     * @return Retorna una lista del Departamento en caso de Exito. En caso de
     * Excepcion Null
     */
    public ArrayList<Departamento> selectByExample(Departamento example) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria myCriteria = session.createCriteria(Departamento.class);
            myCriteria.add(Example.create(example));

            ArrayList<Departamento> result = (ArrayList<Departamento>) myCriteria.list();

            if (result.size() > 0) {
                return result;
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }
}
