/**
 * DocumentoDAO.
 *
 * La capa Modelo: Es la representación específica de la información con la cual
 * el sistema opera, por lo tanto gestiona todos los accesos a dicha
 * información, tanto consultas como actualizaciones, implementando también los
 * privilegios de acceso que se hayan descrito en las especificaciones de la
 * aplicación (lógica de negocio).
 *
 * Envía a la 'vista' aquella parte de la información que en cada momento se le
 * solicita para que sea mostrada (típicamente a un usuario). Las peticiones de
 * acceso o manipulación de información llegan al 'modelo' a través del
 * 'controlador'.
 *
 * La parte DAO del sistema responde a las peticiones de referentes a la Base de
 * Datos cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un registro o insertar en una base de datos).
 *
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Como lo que se busca es caputar la excepcion para poder tratarla y
 * que el Usuario pueda seguir utilizando la aplicacion con total normalidad, se
 * captura la excepcion general "Exception" ya que en todos los casos el
 * tratamiento de la excepcion es el mismo.
 */
package model.dao;

import java.util.ArrayList;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Restrictions;
import model.entities.Documento;
import org.hibernate.Query;
import org.hibernate.Transaction;


public class DocumentoDAO {

    Session session;
    Transaction transaction;

    /**
     * Constructor Vacio.
     */
    public DocumentoDAO() {
    }

    /**
     * Funcion que Abre la Sesion
     * 
     * @return Sesion Abierta
     */
    private Session getSession() {
        try {
            session = HibernateUtil.getSessionFactory().openSession();
            return session;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Funcion que obtiene la transaccion
     * 
     * @return transaccion abierta
     */
    private Transaction beginTransaction() {

        try {
            transaction = session.beginTransaction();
            return transaction;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Insertar Documento
     * 
     * Inserta el Documento en la BD.
     * @param doc Documento a Insertar.
     * @return Retorna True o False docendiendo del Exito del procedimiento.
     */
    public boolean insertDocumento(Documento doc) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.save(doc);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Seleccion del Documento Mediante Numero de Documento
     * 
     * @param id Numero de Documento del Documento a Seleccionar
     * @return Retorna el Documento en caso de Exito. En caso de Excepcion Null
     */
    public Documento selectDocumento(Documento doc) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Documento.class);
            criteria.add(Restrictions.eq("numeroDocumento", doc.getNumeroDocumento()));

            ArrayList<Documento> result = (ArrayList<Documento>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion del Documento Mediante ID Documento
     * 
     * @param id ID del Documento a Seleccionar
     * @return Retorna el Documento en caso de Exito. En caso de Excepcion Null
     */
    public Documento selectDocumento(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Documento.class);
            criteria.add(Restrictions.eq("idDocumento", id));

            ArrayList<Documento> result = (ArrayList<Documento>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Elimina el objeto de la BD
     * 
     * @param doc Documento a Eliminar
     * @return Retorna el Documento en caso de Exito. En caso de Excepcion Null
     */
    public boolean deleteDocumento(Documento doc) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            String hql = "delete from Documento where idDocumento = :idDocumento";

            Query query = session.createQuery(hql);
            query.setInteger("idDocumento", doc.getIdDocumento());

            int rowCount = query.executeUpdate();
            System.out.println("Rows affected: " + rowCount);

            transaction.commit();
            session.close();

            return true;
        } catch (Exception e) {
            session.getTransaction().rollback();
            return false;
        }
    }

    /**
     * Modifica el Documento de la BD
     * 
     * @param doc Documento a Modificar
     * @return Retorna el Documento en caso de Exito. En caso de Excepcion Null
     */
    public boolean updateDocumento(Documento doc) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();


            String hql = "update Documento set  numeroDocumento = :numeroDocumento, tipoDocumento = :tipoDocumento where idDocumento = :idDocumento";

            Query query = session.createQuery(hql);
            query.setString("numeroDocumento", doc.getNumeroDocumento());
            query.setString("tipoDocumento1", doc.getTipoDocumento());
            query.setInteger("idDocumento", doc.getIdDocumento());

            int rowCount = query.executeUpdate();
            System.out.println("Rows affected: " + rowCount);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Selecciona todos los Documentos de la BD
     * 
     * @return Retorna una lista del Documento en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Documento> selectAll() {
        try {
            Criteria myCriteria = this.getSession().createCriteria(Documento.class);
            ArrayList<Documento> result = (ArrayList<Documento>) myCriteria.list();
            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Selecciona los Documentos mediante Un Ejemplo
     * 
     * @param example Documento de Exemplo a seleccionar
     * @return Retorna una lista del Documento en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Documento> selectByExample(Documento example) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria myCriteria = session.createCriteria(Documento.class);
            myCriteria.add(Example.create(example));

            return (ArrayList<Documento>) myCriteria.list();
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }
}
