/**
 * EstadotrabajoDAO.
 *
 * La capa Modelo: Es la representación específica de la información con la cual
 * el sistema opera, por lo tanto gestiona todos los accesos a dicha
 * información, tanto consultas como actualizaciones, implementando también los
 * privilegios de acceso que se hayan descrito en las especificaciones de la
 * aplicación (lógica de negocio).
 *
 * Envía a la 'vista' aquella peste de la información que en cada momento se le
 * solicita para que sea mostrada (típicamente a un usuario). Las peticiones de
 * acceso o manipulación de información llegan al 'modelo' a través del
 * 'controlador'.
 *
 * La peste DAO del sistema responde a las peticiones de referentes a la Base de
 * Datos cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un registro o insertar en una base de datos).
 *
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Como lo que se busca es caputar la excepcion para poder tratarla y
 * que el Usuario pueda seguir utilizando la aplicacion con total normalidad, se
 * captura la excepcion general "Exception" ya que en todos los casos el
 * tratamiento de la excepcion es el mismo.
 */
package model.dao;

import java.util.ArrayList;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Restrictions;
import model.entities.Estadotrabajo;
import model.entities.Usuario;
import org.hibernate.Transaction;


public class EstadotrabajoDAO {

    Session session;
    Transaction transaction;

    /**
     * Constructor Vacio.
     */
    public EstadotrabajoDAO() {
    }
    
    /**
     * Funcion que Abre la Sesion
     * 
     * @return Sesion Abierta
     */
    private Session getSession() {
        try {
            session = HibernateUtil.getSessionFactory().openSession();
            return session;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Funcion que obtiene la transaccion
     * 
     * @return transaccion abierta
     */
    private Transaction beginTransaction() {

        try {
            transaction = session.beginTransaction();
            return transaction;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Insertar Estado Trabajo
     * 
     * Inserta el Estadotrabajo en la BD.
     * @param est Estadotrabajo a Insertar.
     * @return Retorna True o False dependiendo del Exito del procedimiento.
     */
    public boolean insertEstadotrabajo(Estadotrabajo est) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.save(est);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    //selects with criteria
    /**
     * Seleccion del Estadotrabajo Mediante ID Estadotrabajo
     * 
     * @param id ID del Estadotrabajo a Seleccionar
     * @return Retorna el Estadotrabajo en caso de Exito. En caso de Excepcion Null
     */
    public Estadotrabajo selectEstadotrabajo(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Estadotrabajo.class);
            criteria.add(Restrictions.eq("idEstadoTrabajo", id));

            ArrayList<Estadotrabajo> result = (ArrayList<Estadotrabajo>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion del Estadotrabajo Mediante Usuario
     * 
     * @param user Usuario Estadotrabajo a Seleccionar
     * @return Retorna una lista del Estadotrabajo en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Estadotrabajo> selectEstadotrabajo(Usuario user) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Estadotrabajo.class);
            criteria.add(Restrictions.eq("usuario.idUsuario", user.getIdUsuario()));

            ArrayList<Estadotrabajo> result = (ArrayList<Estadotrabajo>) criteria.list();

            return result;

        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Seleccion del Estadotrabajo Mediante Estado Trabajo
     * 
     * @param est Estado del Trabajo del Estadotrabajo a Seleccionar
     * @return Retorna el Estadotrabajo en caso de Exito. En caso de Excepcion Null
     */
    public Estadotrabajo selectEstadotrabajo(Estadotrabajo est) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Estadotrabajo.class);
            criteria.add(Restrictions.eq("trabajo.idTrabajo", est.getTrabajo().getIdTrabajo()));
            criteria.add(Restrictions.eq("usuario.idUsuario", est.getUsuario().getIdUsuario()));

            ArrayList<Estadotrabajo> result = (ArrayList<Estadotrabajo>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Seleccion del Estadotrabajo Mediante Usuario y Estado
     * 
     * @param user Usuario Estadotrabajo a Seleccionar
     * @param estado estado Estadotrabajo a Seleccionar
     * @return Retorna una lista del Estadotrabajo en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Estadotrabajo> selectEstadotrabajo(Usuario user, String estado) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Estadotrabajo.class);
            criteria.add(Restrictions.eq("usuario.idUsuario", user.getIdUsuario()));
            criteria.add(Restrictions.eq("estado", estado));

            ArrayList<Estadotrabajo> result = (ArrayList<Estadotrabajo>) criteria.list();
            return result;

        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Elimina el objeto de la BD
     * 
     * @param est Estadotrabajo a Eliminar
     * @return Retorna el Estadotrabajo en caso de Exito. En caso de Excepcion Null
     */
    public boolean deleteEstadotrabajo(Estadotrabajo est) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.delete(est);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Modifica el Estadotrabajo de la BD
     * 
     * @param est Estadotrabajo a Modificar
     * @return Retorna el Estadotrabajo en caso de Exito. En caso de Excepcion Null
     */
    public boolean updateEstadotrabajo(Estadotrabajo est) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.update(est);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Selecciona todos los Estadotrabajos de la BD
     * 
     * @return Retorna una lista del Estadotrabajo en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Estadotrabajo> selectAll() {
        try {
            Criteria myCriteria = this.getSession().createCriteria(Estadotrabajo.class);
            ArrayList<Estadotrabajo> result = (ArrayList<Estadotrabajo>) myCriteria.list();
            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Selecciona los Estadotrabajos mediante Un Ejemplo
     * 
     * @param example Estadotrabajo de Exemplo a seleccionar
     * @return Retorna una lista del Estadotrabajo en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Estadotrabajo> selectByExample(Estadotrabajo example) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria myCriteria = session.createCriteria(Estadotrabajo.class);
            myCriteria.add(Example.create(example));

            return (ArrayList<Estadotrabajo>) myCriteria.list();
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }
}
