/**
 * IncidenciaDAO.
 *
 * La capa Modelo: Es la representación específica de la información con la cual
 * el sistema opera, por lo tanto gestiona todos los accesos a dicha
 * información, tanto consultas como actualizaciones, implementando también los
 * privilegios de acceso que se hayan descrito en las especificaciones de la
 * aplicación (lógica de negocio).
 *
 * Envía a la 'vista' aquella pincie de la información que en cada momento se le
 * solicita para que sea mostrada (típicamente a un usuario). Las peticiones de
 * acceso o manipulación de información llegan al 'modelo' a través del
 * 'controlador'.
 *
 * La pincie DAO del sistema responde a las peticiones de referentes a la Base de
 * Datos cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un registro o insertar en una base de datos).
 *
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Como lo que se busca es caputar la excepcion para poder tratarla y
 * que el Usuario pueda seguir utilizando la aplicacion con total normalidad, se
 * captura la excepcion general "Exception" ya que en todos los casos el
 * tratamiento de la excepcion es el mismo.
 */
package model.dao;

import java.util.ArrayList;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Restrictions;
import model.entities.Incidencia;
import org.hibernate.Transaction;

public class IncidenciaDAO {

    Session session;
    Transaction transaction;

    /**
     * Constructor Vacio.
     */
    public IncidenciaDAO() {
    }

    /**
     * Funcion que Abre la Sesion
     * 
     * @return Sesion Abierta
     */
    private Session getSession() {
        try {
            session = HibernateUtil.getSessionFactory().openSession();
            return session;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Funcion que obtiene la transaccion
     * 
     * @return transaccion abierta
     */
    private Transaction beginTransaction() {

        try {
            transaction = session.beginTransaction();
            return transaction;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Insertar Incidencia
     * 
     * Inserta la Incidencia en la BD.
     * @param inci Incidencia a Insertar.
     * @return Retorna True o False dependiendo del Exito del procedimiento.
     */
    public boolean insertIncidencia(Incidencia inci) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.save(inci);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    //selects with criteria
    /**
     * Seleccion de la Incidencia Mediante ID Incidencia
     * 
     * @param id ID de la Incidencia a Seleccionar
     * @return Retorna la Incidencia en caso de Exito. En caso de Excepcion Null
     */
    public Incidencia selectIncidencia(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Incidencia.class);
            criteria.add(Restrictions.eq("idIncidencia", id));

            ArrayList<Incidencia> result = (ArrayList<Incidencia>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Elimina el objeto de la BD
     * 
     * @param inci Incidencia a Eliminar
     * @return Retorna la Incidencia en caso de Exito. En caso de Excepcion Null
     */
    public boolean deleteIncidencia(Incidencia inci) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.delete(inci);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Modifica la Incidencia de la BD
     * 
     * @param inci Incidencia a Modificar
     * @return Retorna la Incidencia en caso de Exito. En caso de Excepcion Null
     */
    public boolean updateIncidencia(Incidencia inci) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.update(inci);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Selecciona todos los Incidencias de la BD
     * 
     * @return Retorna una lista de la Incidencia en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Incidencia> selectAll() {
        try {
            Criteria myCriteria = this.getSession().createCriteria(Incidencia.class);
            ArrayList<Incidencia> result = (ArrayList<Incidencia>) myCriteria.list();

            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Selecciona los Incidencias mediante Un Ejemplo
     * 
     * @param example Incidencia de Exemplo a seleccionar
     * @return Retorna una lista de la Incidencia en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Incidencia> selectByExample(Incidencia example) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria myCriteria = session.createCriteria(Incidencia.class);
            myCriteria.add(Example.create(example));

            return (ArrayList<Incidencia>) myCriteria.list();
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    //modifications with HQL
    /**
     * Elimina el objeto de la BD meidante ID
     * 
     * @param id ID de la Incidencia a Eliminar
     * @return Retorna la Incidencia en caso de Exito. En caso de Excepcion Null
     */
    public boolean deleteIncidencia(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            String hql = "delete from Incidencia where idIncidencia = :id";
            Query query = session.createQuery(hql);
            query.setInteger("id", id);

            int rowCount = query.executeUpdate();
            System.out.println("Rows affected: " + rowCount);

            session.getTransaction().commit();
            session.close();

            return true;
        } catch (Exception e) {
            session.getTransaction().rollback();
            return false;
        }
    }
}
