/**
 * LoginDAO.
 *
 * La capa Modelo: Es la representación específica de la información con la cual
 * el sistema opera, por lo tanto gestiona todos los accesos a dicha
 * información, tanto consultas como actualizaciones, implementando también los
 * privilegios de acceso que se hayan descrito en las especificaciones de la
 * aplicación (lógica de negocio).
 *
 * Envía a la 'vista' aquella ploge de la información que en cada momento se le
 * solicita para que sea mostrada (típicamente a un usuario). Las peticiones de
 * acceso o manipulación de información llegan al 'modelo' a través del
 * 'controlador'.
 *
 * La ploge DAO del sistema responde a las peticiones de referentes a la Base de
 * Datos cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un registro o insertar en una base de datos).
 *
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Como lo que se busca es caputar la excepcion para poder tratarla y
 * que el Usuario pueda seguir utilizando la aplicacion con total normalidad, se
 * captura la excepcion general "Exception" ya que en todos los casos el
 * tratamiento de la excepcion es el mismo.
 */
package model.dao;

import java.util.ArrayList;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Restrictions;
import model.entities.Login;
import org.hibernate.Query;
import org.hibernate.Transaction;

public class LoginDAO {

    Session session;
    Transaction transaction;

    /**
     * Constructor Vacio.
     */
    public LoginDAO() {
    }

    /**
     * Funcion que Abre la Sesion
     *
     * @return Sesion Abierta
     */
    private Session getSession() {
        try {
            session = HibernateUtil.getSessionFactory().openSession();
            return session;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Funcion que obtiene la transaccion
     *
     * @return transaccion abierta
     */
    private Transaction beginTransaction() {

        try {
            transaction = session.beginTransaction();
            return transaction;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Insertar Login
     *
     * Inserta el Login en la BD.
     *
     * @param log Login a Insertar.
     * @return Retorna True o False dependiendo del Exito del procedimiento.
     */
    public boolean insertLogin(Login log) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.save(log);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Seleccion del Login Mediante ID Login
     *
     * @param usuario usuario del Login a Seleccionar
     * @return Retorna el Login en caso de Exito. En caso de Excepcion Null
     */
    public Login selectLogin(String usuario) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Login.class);
            criteria.add(Restrictions.eq("usuario", usuario));

            ArrayList<Login> result = (ArrayList<Login>) criteria.list();
            if (result.size() > 0) {

                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    //selects with criteria
    /**
     * Seleccion del Login Mediante ID Login
     *
     * @param id ID del Login a Seleccionar
     * @return Retorna el Login en caso de Exito. En caso de Excepcion Null
     */
    public Login selectLogin(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Login.class);
            criteria.add(Restrictions.eq("idLogin", id));

            ArrayList<Login> result = (ArrayList<Login>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion del Login Mediante Login
     *
     * @param log Login a Seleccionar
     * @return Retorna el Login en caso de Exito. En caso de Excepcion Null
     */
    public Login selectLogin(Login log) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Login.class);
            criteria.add(Restrictions.eq("usuario", log.getUsuario()));
            criteria.add(Restrictions.eq("password", log.getPassword()));

            ArrayList<Login> result = (ArrayList<Login>) criteria.list();
            if (result.size() > 0) {

                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            if (transaction != null) {
                transaction.rollback();
            }
            return null;
        } finally {
            if (session != null) {
                session.close();
            }
        }
    }

    /**
     * Elimina el objeto de la BD
     *
     * @param log Login a Eliminar
     * @return Retorna el Login en caso de Exito. En caso de Excepcion Null
     */
    public boolean deleteLogin(Login log) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            String hql = "delete from Login where idLogin = :idLogin";

            Query query = session.createQuery(hql);
            query.setInteger("idLogin", log.getIdLogin());

            int rowCount = query.executeUpdate();
            System.out.println("Rows affected: " + rowCount);

            session.getTransaction().commit();
            session.close();

            return true;
        } catch (Exception e) {
            session.getTransaction().rollback();
            return false;
        }
    }

    /**
     * Modifica el Login de la BD
     *
     * @param log Login a Modificar
     * @return Retorna el Login en caso de Exito. En caso de Excepcion Null
     */
    public boolean updateLogin(Login log) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            String hql = "update Login set usuario = :usuario, password = :password, dataPassWord = :dataPassWord where idLogin = :idLogin";

            Query query = session.createQuery(hql);
            query.setString("usuario", log.getUsuario());
            query.setString("password", log.getPassword());
            query.setDate("dataPassWord", log.getDataPassWord());
            query.setInteger("idLogin", log.getIdLogin());

            int rowCount = query.executeUpdate();
            System.out.println("Rows affected: " + rowCount);

            transaction.commit();
            session.close();

            return true;
        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Selecciona todos los Logins de la BD
     *
     * @return Retorna una lista del Login en caso de Exito. En caso de
     * Excepcion Null
     */
    public ArrayList<Login> selectAll() {
        try {
            Criteria myCriteria = this.getSession().createCriteria(Login.class);
            ArrayList<Login> result = (ArrayList<Login>) myCriteria.list();

            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Selecciona los Logins mediante Un Ejemplo
     *
     * @param example Login de Exemplo a seleccionar
     * @return Retorna una lista del Login en caso de Exito. En caso de
     * Excepcion Null
     */
    public ArrayList<Login> selectByExample(Login example) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Login.class);
            criteria.add(Example.create(example));

            ArrayList<Login> result = (ArrayList<Login>) criteria.list();

            return result;

        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }
}
