/**
 * PersonaDAO.
 *
 * La capa Modelo: Es la representación específica de la información con la cual
 * el sistema opera, por lo tanto gestiona todos los accesos a dicha
 * información, tanto consultas como actualizaciones, implementando también los
 * privilegios de acceso que se hayan descrito en las especificaciones de la
 * aplicación (lógica de negocio).
 *
 * Envía a la 'vista' aquella parte de la información que en cada momento se le
 * solicita para que sea mostrada (típicamente a un usuario). Las peticiones de
 * acceso o manipulación de información llegan al 'modelo' a través del
 * 'controlador'.
 *
 * La parte DAO del sistema responde a las peticiones de referentes a la Base de
 * Datos cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un registro o insertar en una base de datos).
 *
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Como lo que se busca es caputar la excepcion para poder tratarla y
 * que el Usuario pueda seguir utilizando la aplicacion con total normalidad, se
 * captura la excepcion general "Exception" ya que en todos los casos el
 * tratamiento de la excepcion es el mismo.
 */
package model.dao;

import java.util.ArrayList;
import model.entities.Documento;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Restrictions;
import model.entities.Persona;
import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Transaction;


public class PersonaDAO {

    Session session;
    Transaction transaction;
    DocumentoDAO documentoDAO;

    /**
     * Constructor Vacio.
     */
    public PersonaDAO() {
    }

    /**
     * Funcion que Abre la Sesion
     * 
     * @return Sesion Abierta
     */
    private Session getSession() {
        try {
            session = HibernateUtil.getSessionFactory().openSession();
            return session;
        } catch (Exception e) {
            return null;
        }
    }
    
    /**
     * Funcion que obtiene la transaccion
     * 
     * @return transaccion abierta
     */
    private Transaction beginTransaction() {

        try {
            transaction = session.beginTransaction();
            return transaction;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Insertar la Persona
     * 
     * Inserta la Persona en la BD.
     * @param per Persona a Insertar.
     * @return Retorna True o False dependiendo del Exito del procedimiento.
     */
    public boolean insertPersona(Persona per) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();


            session.save(per.getDocumento());
            session.save(per);

            transaction.commit();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion dla Persona Mediante Documento
     * 
     * @param doc Documento dla Persona a Seleccionar
     * @return Retorna la Persona en caso de Exito. En caso de Excepcion Null
     */
    public Persona selectPersona(Documento doc) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Query query = session.createQuery("from Persona where idDocumento = :idDocumento ");
            query.setParameter("idDocumento", doc.getIdDocumento());

            ArrayList<Persona> result = (ArrayList<Persona>) query.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            session.getTransaction().rollback();
            return null;
        } finally {
            session.getTransaction().commit();
            session.close();
        }
    }

    //selects with criteria
    /**
     * Seleccion dla Persona Mediante ID Persona
     * 
     * @param id ID del Persona a Seleccionar
     * @return Retorna la Persona en caso de Exito. En caso de Excepcion Null
     */
    public Persona selectPersona(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Persona.class);
            criteria.add(Restrictions.eq("idPersona", id));

            ArrayList<Persona> result = (ArrayList<Persona>) criteria.list();
            if (result.size() > 0) {
                Hibernate.initialize(result.get(0).getDocumento());
                return result.get(0);
            } else {
                return null;
            }


        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Elimina el objeto de la BD
     * 
     * @param per Persona a Eliminar
     * @return Retorna la Persona en caso de Exito. En caso de Excepcion Null
     */
    public boolean deletePersona(Persona per) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            String hql = "delete from Persona where idPersona = :idPersona";

            Query query = session.createQuery(hql);
            query.setInteger("idPersona", per.getIdPersona());

            int rowCount = query.executeUpdate();
            System.out.println("Rows affected: " + rowCount);

            session.getTransaction().commit();
            session.close();

            DocumentoDAO docDAO = new DocumentoDAO();
            docDAO.deleteDocumento(per.getDocumento());

            return true;
        } catch (Exception e) {
            session.getTransaction().rollback();
            return false;
        }
    }

    /**
     * Modifica la Persona de la BD
     * 
     * @param per Persona a Modifiper
     * @return Retorna la Persona en caso de Exito. En caso de Excepcion Null
     */
    public boolean updatePersona(Persona per) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();


            String hql = "update Persona set  nombre = :nombre, apellido1 = :apellido1, apellido2 = :apellido2, idioma =:idioma, telefono =:telefono, email =:email where idPersona = :idPersona";

            Query query = session.createQuery(hql);
            query.setString("nombre", per.getNombre());
            query.setString("apellido1", per.getApellido1());
            query.setString("apellido2", per.getApellido2());
            query.setString("idioma", per.getIdioma());
            query.setString("telefono", per.getTelefono());
            query.setString("email", per.getEmail());
            query.setInteger("idPersona", per.getIdPersona());

            int rowCount = query.executeUpdate();
            System.out.println("Rows affected: " + rowCount);

            session.getTransaction().commit();
            session.close();

            return true;
        } catch (Exception e) {
            session.getTransaction().rollback();
            return false;
        }
    }

   /**
     * Selecciona todos los Personas de la BD
     * 
     * @return Retorna una lista dla Persona en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Persona> selectAll() {
        try {
            Criteria myCriteria = this.getSession().createCriteria(Persona.class);
            ArrayList<Persona> result = (ArrayList<Persona>) myCriteria.list();
            if (result.size() > 0) {
                for (Persona l : result) {
                    Hibernate.initialize(l.getDocumento());
                }
            }
            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Selecciona los Personas mediante Un Ejemplo
     * 
     * @param example Persona de Exemplo a seleccionar
     * @return Retorna una lista de la Persona en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Persona> selectByExample(Persona example) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria myCriteria = session.createCriteria(Persona.class);
            myCriteria.add(Example.create(example));

            ArrayList<Persona> result = (ArrayList<Persona>) myCriteria.list();

            if (result.size() > 0) {
                for (Persona l : result) {
                    Hibernate.initialize(l.getDocumento());
                }


            }
            return result;

        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }
}
