/**
 * PlantillaDAO.
 *
 * La capa Modelo: Es la representación específica de la información con la cual
 * el sistema opera, por lo tanto gestiona todos los accesos a dicha
 * información, tanto consultas como actualizaciones, implementando también los
 * privilegios de acceso que se hayan descrito en las especificaciones de la
 * aplicación (lógica de negocio).
 *
 * Envía a la 'vista' aquella parte de la información que en cada momento se le
 * solicita para que sea mostrada (típicamente a un usuario). Las peticiones de
 * acceso o manipulación de información llegan al 'modelo' a través del
 * 'controlador'.
 *
 * La parte DAO del sistema responde a las peticiones de referentes a la Base de
 * Datos cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un registro o insertar en una base de datos).
 *
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Como lo que se busca es caputar la excepcion para poder tratarla y
 * que el Usuario pueda seguir utilizando la aplicacion con total normalidad, se
 * captura la excepcion general "Exception" ya que en todos los casos el
 * tratamiento de la excepcion es el mismo.
 */
package model.dao;

import java.util.ArrayList;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Restrictions;
import model.entities.Plantilla;
import org.hibernate.Transaction;

public class PlantillaDAO {

    Session session;
    Transaction transaction;

    /**
     * Constructor Vacio.
     */
    public PlantillaDAO() {
    }

    /**
     * Funcion que Abre la Sesion
     *
     * @return Sesion Abierta
     */
    private Session getSession() {
        try {
            session = HibernateUtil.getSessionFactory().openSession();;
            return session;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Funcion que obtiene la transaccion
     *
     * @return transaccion abierta
     */
    private Transaction beginTransaction() {

        try {
            transaction = session.beginTransaction();
            return transaction;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Insertar Cartel
     *
     * Inserta la Plantilla en la BD.
     *
     * @param plan Plantilla a Insertar.
     * @return Retorna True o False dependiendo del Exito del procedimiento.
     */
    public boolean insertPlantilla(Plantilla plan) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.save(plan);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    //selects with criteria
    /**
     * Seleccion de la Plantilla Mediante ID Plantilla
     *
     * @param id ID de la Plantilla a Seleccionar
     * @return Retorna la Plantilla en caso de Exito. En caso de Excepcion Null
     */
    public Plantilla selectPlantilla(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Plantilla.class);
            criteria.add(Restrictions.eq("idPlantilla", id));

            ArrayList<Plantilla> result = (ArrayList<Plantilla>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion de la Plantilla Mediante ID Plantilla
     *
     * @param plan Plantilla a seleccionar
     * @return Retorna la Plantilla en caso de Exito. En caso de Excepcion Null
     */
    public Plantilla selectPlantilla(Plantilla plan) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Plantilla.class);
            criteria.add(Restrictions.eq("tipo", plan.getTipo()));
            criteria.add(Restrictions.eq("tamano", plan.getTamano()));

            ArrayList<Plantilla> result = (ArrayList<Plantilla>) criteria.list();
            if (result.size() > 0) {
                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Elimina el objeto de la BD
     *
     * @param plan Plantilla a Eliminar
     * @return Retorna la Plantilla en caso de Exito. En caso de Excepcion Null
     */
    public boolean deletePlantilla(Plantilla plan) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.delete(plan);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Modifica la Plantilla de la BD
     *
     * @param plan Plantilla a Modifiplan
     * @return Retorna la Plantilla en caso de Exito. En caso de Excepcion Null
     */
    public boolean updatePlantilla(Plantilla plan) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.update(plan);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Selecciona todos los Plantillas de la BD
     *
     * @return Retorna una lista de la Plantilla en caso de Exito. En caso de
     * Excepcion Null
     */
    public ArrayList<Plantilla> selectAll() {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria myCriteria = session.createCriteria(Plantilla.class);
            ArrayList<Plantilla> result = (ArrayList<Plantilla>) myCriteria.list();

            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Selecciona los Plantillas mediante Un Ejemplo
     *
     * @param example Plantilla de Exemplo a seleccionar
     * @return Retorna una lista de la Plantilla en caso de Exito. En caso de
     * Excepcion Null
     */
    public ArrayList<Plantilla> selectByExample(Plantilla example) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria myCriteria = session.createCriteria(Plantilla.class);
            myCriteria.add(Example.create(example));

            ArrayList<Plantilla> result = (ArrayList<Plantilla>) myCriteria.list();

            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }
}
