/**
 * TrabajoDAO.
 *
 * La capa Modelo: Es la representación específica de la información con la cual
 * el sistema opera, por lo tanto gestiona todos los accesos a dicha
 * información, tanto consultas como actualizaciones, implementando también los
 * privilegios de acceso que se hayan descrito en las especificaciones de la
 * aplicación (lógica de negocio).
 *
 * Envía a la 'vista' aquella parte de la información que en cada momento se le
 * solicita para que sea mostrada (típicamente a un usuario). Las peticiones de
 * acceso o manipulación de información llegan al 'modelo' a través del
 * 'controlador'.
 *
 * La parte DAO del sistema responde a las peticiones de referentes a la Base de
 * Datos cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un registro o insertar en una base de datos).
 *
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Como lo que se busca es caputar la excepcion para poder tratarla y
 * que el Usuario pueda seguir utilizando la aplicacion con total normalidad, se
 * captura la excepcion general "Exception" ya que en todos los casos el
 * tratamiento de la excepcion es el mismo.
 */
package model.dao;

import java.util.ArrayList;
import java.util.Iterator;
import model.entities.Cartel;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Restrictions;
import model.entities.Trabajo;
import org.hibernate.Hibernate;
import org.hibernate.Transaction;

public class TrabajoDAO {

    Session session;
    Transaction transaction;

    /**
     * Constructor Vacio.
     */
    public TrabajoDAO() {
    }

    /**
     * Funcion que Abre la Sesion
     *
     * @return Sesion Abierta
     */
    private Session getSession() {
        try {
            session = HibernateUtil.getSessionFactory().openSession();
            return session;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Funcion que obtiene la transaccion
     *
     * @return transaccion abierta
     */
    private Transaction beginTransaction() {

        try {
            transaction = session.beginTransaction();
            return transaction;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Insertar Plantilla
     *
     * Inserta el Trabajo en la BD.
     *
     * @param tra Trabajo a Insertar.
     * @return Retorna True o False dependiendo del Exito del procedimiento.
     */
    public boolean insertTrabajo(Trabajo tra) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.save(tra);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Seleccion del Trabajo Mediante ID Trabajo
     *
     * @param id ID del Trabajo a Seleccionar
     * @return Retorna el Trabajo en caso de Exito. En caso de Excepcion Null
     */
    public Trabajo selectTrabajo(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Trabajo.class);
            criteria.add(Restrictions.eq("idTrabajo", id));

            ArrayList<Trabajo> result = (ArrayList<Trabajo>) criteria.list();
            if (result.size() > 0) {
                Hibernate.initialize(result.get(0).getCarteles());


                Iterator<Cartel> it = result.get(0).getCarteles().iterator();

                while (it.hasNext()) {

                    Cartel tra = it.next();
                    Hibernate.initialize(tra);
                    Hibernate.initialize(tra.getArticulo());
                    Hibernate.initialize(tra.getPlantilla());
                }

                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion del Trabajo Mediante ID Trabajo
     *
     * @param tra ID del aTrabajo a Seleccionar
     * @return Retorna el Trabajo en caso de Exito. En caso de Excepcion Null
     */
    public Trabajo selectTrabajo(Trabajo tra) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Trabajo.class);
            criteria.add(Restrictions.eq("descripcion", tra.getDescripcion()));

            ArrayList<Trabajo> result = (ArrayList<Trabajo>) criteria.list();
            if (result.size() > 0) {
                Hibernate.initialize(result.get(0).getCarteles());

                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Elimina el Trabajo de la BD
     *
     * @param tra Trabajo a Eliminar
     * @return Retorna el Trabajo en caso de Exito. En caso de Excepcion Null
     */
    public boolean deleteTrabajo(Trabajo tra) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.delete(tra);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Modifica el Trabajo de la BD
     *
     * @param tra Trabajo a Modifitra
     * @return Retorna el Trabajo en caso de Exito. En caso de Excepcion Null
     */
    public boolean updateTrabajo(Trabajo tra) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.update(tra);

            transaction.commit();
            session.close();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        }
    }

    /**
     * Selecciona todos los Trabajos de la BD
     *
     * @return Retorna una lista del Trabajo en caso de Exito. En caso de
     * Excepcion Null
     */
    public ArrayList<Trabajo> selectAll() {
        try {
            Criteria myCriteria = this.getSession().createCriteria(Trabajo.class);
            ArrayList<Trabajo> result = (ArrayList<Trabajo>) myCriteria.list();

            if (result.size() > 0) {
                for (Trabajo l : result) {
                    Hibernate.initialize(l.getCarteles());
                }
            }
            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Selecciona los Trabajos mediante Un Ejemplo
     *
     * @param example Trabajo de Exemplo a seleccionar
     * @return Retorna una lista del Trabajo en caso de Exito. En caso de
     * Excepcion Null
     */
    public ArrayList<Trabajo> selectByExample(Trabajo example) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria myCriteria = session.createCriteria(Trabajo.class);
            myCriteria.add(Example.create(example));

            ArrayList<Trabajo> result = (ArrayList<Trabajo>) myCriteria.list();

            if (result.size() > 0) {
                for (Trabajo l : result) {
                    Hibernate.initialize(l.getCarteles());
                }
            }
            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }
}
