/**
  * UsuarioDAO.
  * 
  * La capa Modelo: Es la representación específica de la información con la cual el sistema opera, 
  * por lo tanto gestiona todos los accesos a dicha información, tanto consultas como actualizaciones, 
  * implementando también los privilegios de acceso que se hayan descrito en las especificaciones de la aplicación (lógica de negocio). 
  * 
  * Envía a la 'vista' aquella parte de la información que en cada momento se le solicita para que sea mostrada (típicamente a un usuario). 
  * Las peticiones de acceso o manipulación de información llegan al 'modelo' a través del 'controlador'.

  * La parte DAO del sistema responde a las peticiones de referentes a la Base de Datos cuando se hace alguna solicitud sobre la información
  * (por ejemplo, editar un registro o insertar en una base de datos). 
  * 
  * 
  * @author JuanRamon Segura Ponce
  *
  * @version v.1.0
  * @throws Como lo que se busca es caputar la excepcion para poder tratarla y que el Usuario pueda seguir utilizando
  * la aplicacion con total normalidad, se captura la excepcion general "Exception" ya que en todos los casos el tratamiento
  * de la excepcion es el mismo.
  */

package model.dao;

import java.util.ArrayList;
import model.entities.Departamento;
import model.entities.Login;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Restrictions;
import model.entities.Usuario;
import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Transaction;


public class UsuarioDAO {

    Session session;
    Transaction transaction;
    DocumentoDAO documentoDAO;

    /**
     * Constructor Vacio
     */
    public UsuarioDAO() {
    }

        /**
     * Funcion que Abre la Sesion
     * 
     * @return Sesion Abierta
     */
    
    private Session getSession() {
        try {
            session = HibernateUtil.getSessionFactory().openSession();
            return session;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Funcion que obtiene la transaccion
     * 
     * @return transaccion abierta
     */
    private Transaction beginTransaction() {

        try {
            transaction = session.beginTransaction();
            return transaction;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Insertar Usuario
     * 
     * Inserta el Usuario en la BD.
     * @param user Usuario a Insertar.
     * @return Retorna True o False dependiendo del Exito del procedimiento.
     */
    public boolean insertUsuario(Usuario user) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            session.save(user.getLogin());
            session.save(user);

            transaction.commit();

            return true;

        } catch (Exception e) {
            transaction.rollback();
            return false;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion del Usuario Mediante Login
     * 
     * @param log Login del Usuario a Seleccionar
     * @return Retorna el Usuario en caso de Exito. En caso de Excepcion Null
     */
    public Usuario selectUsuario(Login log) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Usuario.class);
            criteria.add(Restrictions.eq("login.idLogin", log.getIdLogin()));

            ArrayList<Usuario> result = (ArrayList<Usuario>) criteria.list();

            if (result.size() > 0) {
                Hibernate.initialize(result.get(0).getLogin());
                Hibernate.initialize(result.get(0).getPersona());
                Hibernate.initialize(result.get(0).getDepartamento());

                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion del Usuario Mediante ID Usuario
     * 
     * @param id ID del Usuario a Seleccionar
     * @return Retorna el Usuario en caso de Exito. En caso de Excepcion Null
     */
    public Usuario selectUsuario(Integer id) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Usuario.class);
            criteria.add(Restrictions.eq("idUsuario", id));

            ArrayList<Usuario> result = (ArrayList<Usuario>) criteria.list();

            if (result.size() > 0) {
                Hibernate.initialize(result.get(0).getLogin());
                Hibernate.initialize(result.get(0).getPersona());
                Hibernate.initialize(result.get(0).getDepartamento());

                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion del Usuario Mediante Usuario
     * 
     * @param user Usuario a seleccionar
     * @return Retorna el Usuario en caso de Exito. En caso de Excepcion Null
     */
    public Usuario selectUsuario(Usuario user) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Usuario.class);
            criteria.add(Restrictions.eq("idUsuario", user.getIdUsuario()));

            ArrayList<Usuario> result = (ArrayList<Usuario>) criteria.list();

            if (result.size() > 0) {
                Hibernate.initialize(result.get(0).getLogin());
                Hibernate.initialize(result.get(0).getPersona());
                Hibernate.initialize(result.get(0).getDepartamento());

                return result.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Seleccion del Usuario Mediante Departamento
     * 
     * @param user usertel a seleccionar
     * @return Retorna Una lista del Usuario en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Usuario> selectUsuario(Departamento dep) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria criteria = session.createCriteria(Usuario.class);
            criteria.add(Restrictions.eq("departamento.idDepartamento", dep.getIdDepartamento()));

            ArrayList<Usuario> result = (ArrayList<Usuario>) criteria.list();

            if (result.size() > 0) {
                for (Usuario l : result) {
                    Hibernate.initialize(l.getLogin());
                    Hibernate.initialize(l.getPersona());
                    Hibernate.initialize(l.getDepartamento());
                }
            }

            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Elimina el objeto de la BD
     * 
     * @param user Usuario a Eliminar
     * @return Retorna el Usuario en caso de Exito. En caso de Excepcion Null
     */
    public boolean deleteUsuario(Usuario user) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            String hql = "delete from Usuario where idUsuario = :idUsuario";

            Query query = session.createQuery(hql);
            query.setInteger("idUsuario", user.getIdUsuario());

            int rowCount = query.executeUpdate();
            System.out.println("Rows affected: " + rowCount);

            transaction.commit();
            session.close();

            LoginDAO logDAO = new LoginDAO();
            logDAO.deleteLogin(user.getLogin());

            return true;
        } catch (Exception e) {
            session.getTransaction().rollback();
            return false;
        }
    }

    /**
     * Modifica el Usuario de la BD
     * 
     * @param user Usuario a Modifiuser
     * @return Retorna el Usuario en caso de Exito. En caso de Excepcion Null
     */
    public boolean updateUsuario(Usuario user) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            String hql = "update Usuario set tipoUsuario = :tipoUsuario, estadoEnSistema = :estadoEnSistema, emailUsuario = :emailUsuario, telefonoUsuario =:telefonoUsuario where idUsuario = :idUsuario";

            Query query = session.createQuery(hql);
            query.setString("tipoUsuario", user.getTipoUsuario());
            query.setString("estadoEnSistema", user.getEstadoEnSistema());
            query.setString("emailUsuario", user.getEmailUsuario());
            query.setString("telefonoUsuario", user.getTelefonoUsuario());
            query.setInteger("idUsuario", user.getIdUsuario());

            int rowCount = query.executeUpdate();
            System.out.println("Rows affected: " + rowCount);

            DepartamentoDAO depDAO = new DepartamentoDAO();
            depDAO.updateDepartamento(user.getDepartamento());

            transaction.commit();
            session.close();

            return true;
        } catch (Exception e) {
            session.getTransaction().rollback();
            return false;
        }
    }

    /**
     * Selecciona todos los Usuarios de la BD
     * 
     * @return Retorna una lista del Usuario en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Usuario> selectAll() {
        try {
            Criteria myCriteria = this.getSession().createCriteria(Usuario.class);
            ArrayList<Usuario> result = (ArrayList<Usuario>) myCriteria.list();
            if (result.size() > 0) {
                for (Usuario l : result) {
                    Hibernate.initialize(l.getLogin());
                    Hibernate.initialize(l.getPersona());
                    Hibernate.initialize(l.getDepartamento());
                }
            }

            return result;
        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }

    /**
     * Selecciona los Usuarios mediante Un Ejemplo
     * 
     * @param example Usuario de Exemplo a seleccionar
     * @return Retorna una lista del Usuario en caso de Exito. En caso de Excepcion Null
     */
    public ArrayList<Usuario> selectByExample(Usuario example) {
        try {
            session = this.getSession();
            transaction = this.beginTransaction();

            Criteria myCriteria = session.createCriteria(Usuario.class);
            myCriteria.add(Example.create(example));

            ArrayList<Usuario> result = (ArrayList<Usuario>) myCriteria.list();

            if (result.size() > 0) {
                for (Usuario l : result) {
                    Hibernate.initialize(l.getLogin());
                    Hibernate.initialize(l.getPersona());
                    Hibernate.initialize(l.getDepartamento());
                }
            }
            return result;

        } catch (Exception e) {
            transaction.rollback();
            return null;
        } finally {
            session.close();
        }
    }
}
