/**
 * Cartel.
 *
 * Es la entidad que se encarga de encapsular toda la informacion referente al
 * CARTEL.
 *
 * Los ENTITIES se usan para encapsular varios objetos en un único objeto (la
 * vaina o Bean en inglés), para hacer uso de un solo objeto en lugar de varios
 * más simples, son clases simples y que no dependen de un framework en
 * especial.
 *
 * Para funcionar como una clase JavaBean, una clase debe obedecer ciertas
 * convenciones sobre nomenclatura de métodos, construcción y comportamiento.
 *
 * Estas convenciones permiten tener herramientas que puedan utilizar,
 * reutilizar, sustituir y conectar JavaBeans.
 *
 * Sus propiedades deben ser accesibles mediante métodos get y set que siguen
 * una convención de nomenclatura estándar y debe ser serializable.
 *
 * Dentro de un JavaBean podemos distinguir tres partes: Propiedades: Los
 * atributos que contiene. Métodos: Se establecen los métodos get y set para
 * acceder y modificar los atributos. Eventos: Permiten comunicar con otros
 * JavaBeans.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws --> No Throws
 */

package model.entities;

import java.text.Collator;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;

public class Cartel implements java.io.Serializable {

    
    //Example: Collections.sort(listaCartel,car.COMPARADOR_POR_PLANTILLA); 
    
    /**
     * Comparador tipo de plantilla para la ordenacion de colecciones.
     */
    public static Comparator COMPARADOR_POR_PLANTILLA = new Comparator() {
        private Collator theC = Collator.getInstance();

        @Override
        public int compare(Object o1, Object o2) {
            Cartel car1 = (Cartel) o1;
            Cartel car2 = (Cartel) o2;
            return theC.compare(car1.getPlantilla().toStr(), car2.getPlantilla().toStr());
        }
    };
    
    /**
     * Comparador idCartel para la ordenacion de colecciones.
     */
    public static Comparator COMPARADOR_POR_ID = new Comparator() {
        @Override
        public int compare(Object o1, Object o2) {
            Cartel car1 = (Cartel) o1;
            Cartel car2 = (Cartel) o2;
            return car1.getIdCartel() - car2.getIdCartel();
        }
    };
    
    
    private Integer idCartel;
    private Articulo articulo;
    private Plantilla plantilla;
    private Date dataIni;
    private Date dataFi;

    /**
     * Constructor vacio.
     */
    public Cartel() {
    }

    /**
     * Constructor con los parametros minimos "NOT NULL"
     * 
     * @param articulo Propiedad del Cartel.
     * @param plantilla Propiedad del Cartel.
     */
    public Cartel(Articulo articulo, Plantilla plantilla) {
        this.articulo = articulo;
        this.plantilla = plantilla;
    }

    /**
     * Constructor con todos los parametros del Objeto.
     * 
     * @param articulo Propiedad del Cartel.
     * @param plantilla Propiedad del Cartel.
     * @param dataIni Propiedad del Cartel.
     * @param dataFi Propiedad del Cartel.
     */
    public Cartel(Articulo articulo, Plantilla plantilla, Date dataIni, Date dataFi) {
        this.articulo = articulo;
        this.plantilla = plantilla;
        this.dataIni = dataIni;
        this.dataFi = dataFi;
    }

    /**
     * Getter de la propiedad Id Cartel
     * 
     * @return ID del Cartel (idCartel)
     */
    public Integer getIdCartel() {
        return this.idCartel;
    }

    /**
     * Setter de la propiedad idCartel
     * 
     * @param idCartel Propiedad del Cartel.
     */
    public void setIdCartel(Integer idCartel) {
        this.idCartel = idCartel;
    }

    /**
     * Getter de la propiedad Articulo
     *  
     * @return Retorna la propiedad.
     */
    public Articulo getArticulo() {
        return this.articulo;
    }

    /**
     * Setter de la propiedad articulo (objeto)
     * 
     * @param articulo Propiedad del Cartel.
     */
    public void setArticulo(Articulo articulo) {
        this.articulo = articulo;
    }

    /**
     * Getter de la propiedad Plantilla
     * 
     * @return Retorna la propiedad.
     */
    public Plantilla getPlantilla() {
        return this.plantilla;
    }

    /**
     * Setter de la propiedad plantilla (objeto)
     * 
     * @param plantilla Propiedad del Cartel.
     */
    public void setPlantilla(Plantilla plantilla) {
        this.plantilla = plantilla;
    }

    /**
     * Getter de la propiedad Data Inicio 
     * 
     * @return Retorna la propiedad.
     */
    public Date getDataIni() {
        return this.dataIni;
    }

    /**
     * Setter de la propiedad Data Inicial
     * 
     * @param dataIni Propiedad del Cartel.
     */
    public void setDataIni(Date dataIni) {
        this.dataIni = dataIni;
    }

    /**
     * Getter de la propiedad Data Final
     * 
     * @return Retorna la propiedad.
     */
    public Date getDataFi() {
        return this.dataFi;
    }

    /**
     * Setter de la propiedad Data Final
     * 
     * @param dataFi Propiedad del Cartel.
     */
    public void setDataFi(Date dataFi) {
        this.dataFi = dataFi;
    }

    /**
     * Coloca todos los valores de las propiedades a Null.
     */
    public void reset() {
        this.idCartel = null;
        this.articulo = null;
        this.plantilla = null;
        this.dataIni = null;
        this.dataFi = null;
    }

    

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Cartel other = (Cartel) obj;
        if (this.idCartel != other.idCartel && (this.idCartel == null || !this.idCartel.equals(other.idCartel))) {
            return false;
        }
        if (this.articulo != other.articulo && (this.articulo == null || !this.articulo.equals(other.articulo))) {
            return false;
        }
        if (this.plantilla != other.plantilla && (this.plantilla == null || !this.plantilla.equals(other.plantilla))) {
            return false;
        }
        if (this.dataIni != other.dataIni && (this.dataIni == null || !this.dataIni.equals(other.dataIni))) {
            return false;
        }
        if (this.dataFi != other.dataFi && (this.dataFi == null || !this.dataFi.equals(other.dataFi))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        String str = "idCartel=" + idCartel + ", " + plantilla.toStr() + ","+articulo.toStr() ;
        DateFormat df = DateFormat.getDateInstance();
        
       if (dataIni != null) {
            String ini = df.format(dataIni);
            str = str + ", dataIni" + ini;
        }

        if (dataFi != null) {
            String fin = df.format(dataFi);
            str = str + ", dataFi" + fin;
        }
        return str;
    }

    /*
    public String toStr() {
        DateFormat df = DateFormat.getDateInstance();
        String ini = new String();
        String fin = new String();

        String str = idCartel + " " + plantilla.toStr() + " " + articulo.toStr();

        if (dataIni != null) {
            ini = df.format(dataIni);
            str = str + " " + ini;
        }

        if (dataFi != null) {
            fin = df.format(dataFi);
            str = str + " " + fin;
        }
        return str;
    }*/
}
